@@ -362,8 +362,8 @@ static int pljs_execute_params(const char *sql, JSValue params,
     JSValue param = JS_GetPropertyUint32(ctx, params, i);
     bool is_null;
 
-    values[i] = pljs_jsvalue_to_datum(param, parstate.param_types[i], ctx, NULL,
-                                      &is_null);
+    values[i] = pljs_jsvalue_to_datum(parstate.param_types[i], param, &is_null,
+                                      ctx, NULL);
 
     JS_FreeValue(ctx, param);
   }
@@ -441,8 +441,8 @@ static JSValue pljs_plan_execute(JSContext *ctx, JSValueConst this_val,
     bool is_null;
 
     values[i] = pljs_jsvalue_to_datum(
-        param, plan->parstate ? plan->parstate->param_types[i] : 0, ctx, NULL,
-        &is_null);
+        plan->parstate ? plan->parstate->param_types[i] : 0, param, &is_null,
+        ctx, NULL);
 
     JS_FreeValue(ctx, param);
   }
@@ -739,8 +739,8 @@ static JSValue pljs_plan_cursor(JSContext *ctx, JSValueConst this_val, int argc,
     bool is_null;
 
     values[i] = pljs_jsvalue_to_datum(
-        param, plan->parstate ? plan->parstate->param_types[i] : 0, ctx, NULL,
-        &is_null);
+        plan->parstate ? plan->parstate->param_types[i] : 0, param, &is_null,
+        ctx, NULL);
   }
 
   PG_TRY();
@@ -1069,14 +1069,20 @@ static JSValue pljs_return_next(JSContext *ctx, JSValueConst this_val, int argc,
       return js_throw("field name / property name mismatch", ctx);
     }
 
-    bool is_null = false;
-    pljs_jsvalue_to_record(argv[0], NULL, ctx, &is_null, retstate->tuple_desc,
-                           retstate->tuple_store_state);
+    bool *nulls = (bool *)palloc0(sizeof(bool) * retstate->tuple_desc->natts);
+    Datum *values = pljs_jsvalue_to_datums(NULL, argv[0], &nulls,
+                                           retstate->tuple_desc, ctx);
+
+    tuplestore_putvalues(retstate->tuple_store_state, retstate->tuple_desc,
+                         values, nulls);
+
+    pfree(nulls);
+    pfree(values);
   } else {
     bool is_null = false;
-    Datum result = pljs_jsvalue_to_datum(
-        argv[0], TupleDescAttr(retstate->tuple_desc, 0)->atttypid, ctx, NULL,
-        &is_null);
+    Datum result =
+        pljs_jsvalue_to_datum(TupleDescAttr(retstate->tuple_desc, 0)->atttypid,
+                              argv[0], &is_null, ctx, NULL);
     tuplestore_putvalues(retstate->tuple_store_state, retstate->tuple_desc,
                          &result, &is_null);
   }
@@ -1372,8 +1378,8 @@ static JSValue pljs_window_get_func_arg_in_partition(JSContext *ctx,
     return JS_UNDEFINED;
   }
 
-  return pljs_datum_to_jsvalue(res, storage->function->argtypes[argno], ctx,
-                               false);
+  return pljs_datum_to_jsvalue(storage->function->argtypes[argno], res, isnull,
+                               true, ctx);
 }
 
 static JSValue pljs_window_get_func_arg_in_frame(JSContext *ctx,
@@ -1418,8 +1424,8 @@ static JSValue pljs_window_get_func_arg_in_frame(JSContext *ctx,
   if (isout) {
     return JS_UNDEFINED;
   }
-  return pljs_datum_to_jsvalue(res, storage->function->argtypes[argno], ctx,
-                               false);
+  return pljs_datum_to_jsvalue(storage->function->argtypes[argno], res, isnull,
+                               true, ctx);
 }
 
 static JSValue pljs_window_get_func_arg_current(JSContext *ctx,
@@ -1450,8 +1456,8 @@ static JSValue pljs_window_get_func_arg_current(JSContext *ctx,
   }
   PG_END_TRY();
 
-  return pljs_datum_to_jsvalue(res, storage->function->argtypes[argno], ctx,
-                               false);
+  return pljs_datum_to_jsvalue(storage->function->argtypes[argno], res, isnull,
+                               true, ctx);
 }
 
 static JSValue pljs_window_object_to_string(JSContext *ctx,
